import random
from scipy.special import softmax
import numpy as np
import multiprocessing as mp
from matplotlib.patches import Ellipse
from lmz import *
from ubergauss.tools import cartesian_product, xmap, maxnum
from sklearn import ensemble

'''
BORE optimizes by just using prediction rations for the pos/negative classes on a classifier that can
output probabilities...
'''


def get_grid(space):
        X = [np.linspace(a,b,100) for a,b in space]
        X = cartesian_product(X)
        return X

def get1index(clf):
    if clf.classes_[0] == 1:
        return 0
    else:
        return 1



class BAY:
    '''
    we should discuss how the space is defined:
    space = [(mi,ma),...],{a:(mi,ma)}
    '''
    def __init__(self,f, space, n_init =10):

        self.f = f
        self.space = space
        self.params = []
        self.values = []

        if n_init:
            pt  = [self.randomsample() for i in range(n_init)]
            scores = xmap(f,pt)
            self.register (pt,scores)



    def register(self,params:list, values: list):
        self.params+=params
        self.values+= values

    def suggest(self, n= 5):
        pass

    def randomsample(self):
        # returns 1 random parameter set
        return [np.random.uniform(a,b) for a,b in self.space]


    def minimize(self,n_iter=20, n_jobs = 5):
        while n_iter > 0:
            args = self.suggest(n_jobs)
            values = xmap(self.f,args)
            self.register(args, values)
            n_iter -= n_jobs

    ################
    # Fitting..
    ##################
    def fit(self, sample=0, draw=False):
        assert len(self.params) > 1, 'do some random guessing first'

        if 'model'  in self.__dict__:
            #self.space= [(x.min(),x.max()) for x in self.model[0].T]
            pass

        fpos = ensemble.RandomForestClassifier(n_estimators=50)
        fneg = ensemble.RandomForestClassifier(n_estimators=50)

        #fpos = ensemble.RandomForestRegressor(n_estimators=5)
        #fneg = ensemble.RandomForestRegressor(n_estimators=5)

        values = np.array(self.values)
        param = np.array(self.params)


        argsrt = np.argsort(self.values) # smallest to highest
        cut = max(int(len(self.values)/4),1)
        values = np.zeros(len(self.values))
        values[argsrt[:cut]] = 1

        #tools.binarize(self.values,.25) -> i  should be using this :)
        fpos.fit(param, values)
        #vinf = (values+1)%2
        #fneg.fit(param, vinf)


        #for p,v in zip(param,values): print(f"{p} {myf(p)} {v}")

        '''
        get prediction for the whole space
        '''
        X = get_grid(self.space)


        #Y1,Y2 = fpos.predict_proba(X)[:,get1index(fpos)], fneg.predict_proba(X)[:,get1index(fneg)]
        pos = get1index(fpos)
        Y1,Y2 = fpos.predict_proba(X)[:,pos], fpos.predict_proba(X)[:, 0 if pos == 1 else 0]



        #Y1,Y2 = fpos.predict(X), fneg.predict(X)
        # TODO  Y2 being zero is a problem..
        #  with obvious solutions y2==0 -> picj definitely from there
        Y1f = Y1.flatten()
        Y2f = Y2.flatten()
        ie = Y1f/Y2f
        ie = np.where(ie != np.inf, ie, Y1f * maxnum(ie)*2)
        #ie = np.nan_to_num(ie,posinf=i, nan = i, neginf = i)
        ieNU = ie
        XNU = X

        ieNU = ieNU*ieNU.T

        # ieNU = np.log(ie)
        # XNU = X[ieNU > 0]
        # ieNU = ieNU[ieNU > 0]

        cut = np.median(ieNU)
        XNU = X[ieNU > cut]
        ieNU = ieNU[ieNU > cut]
        '''
        #ieNU = -ieNU
        #ieNU/=max(ieNU)
        #ieNU = ieNU*ieNU.T
        '''

        self.model = XNU, ieNU  # coordinates, pribabilities

        probe_here = 0
        if sample:
            probe_here = self.sample(sample)

        if draw:
            xylim = self.space
            plt.figure(figsize=(9,6))
            plt.subplot(231)
            d2plot(np.array(self.params),self.values, title = 'sampled data', xylim = xylim)
            plt.subplot(232)
            d2plot(X,Y1, title = 'POS', xylim = xylim)
            plt.subplot(233)
            d2plot(X,Y2, title = 'NEG', xylim = xylim)
            plt.subplot(234)
            d2plot(X,ie, title = 'pos/neg', xylim = xylim)
            plt.subplot(235)
            d2plot(XNU,ieNU, title = 'sampleweight', xylim = xylim)
            if sample:
                plt.subplot(236)
                d2plot(np.array(probe_here),y='r', title = 'new samples', xylim = xylim)
            plt.show()
            plt.close()
        return probe_here


    def sample(self, num):
        # so we sample,,, ie are the weights...
        #return random.choices(self.model[0], self.model[1])[0]
        X,w = self.model
        sample = np.random.choice(Range(X),num,replace=False, p=w/sum(w))
        return [X[r] for r in sample]


    def bestsuggestion(self):
        # so we sample,,, ie are the weights...
        #return random.choices(self.model[0], self.model[1])[0]
        X,w = self.model
        sample = np.argmax(w)
        return X[sample]




import matplotlib
matplotlib.use("module://matplotlib-sixel")
import matplotlib.pyplot as plt

def d2plot(X,y, done = False, title = None,xylim=None):
    pl=plt.scatter(X[:,0],X[:,1], c= y,s=9)
    plt.xlim(*xylim[0])
    plt.ylim(*xylim[1])
    plt.colorbar(pl)
    if title:
        plt.title(title)
    if done:
        plt.show()
        plt.close()




def myf(args):
    #return (args[0]**2)+(args[1]**2)
    return min(((args[0]+30)**2)+((args[1]+90)**2), ((args[0]-30)**2)+((args[1]-60)**2))




if __name__ == "__main__":
    opti = BAY(myf,[(-100,100),(-100,100)],n_init = 10)


    X = get_grid(opti.space)
    plt.scatter(X[:,0],X[:,1], c=Map(myf, X), s=9)
    plt.show();plt.close()

    if True:
        for i in range(6):
            pt = opti.fit(draw=True, sample = 5)
            score = [myf(x) for x in pt]
            opti.register(pt,score)

    if False:
        for i in range(10):
            pt  = opti.bestsuggestion()
            score = [myf(pt)]
            opti.register([pt],score)
            opti.fit(draw=True)



from multiprocessing import Pool
_func = None
def worker_init(func):
  global _func
  _func = func
def worker(x):
  return _func(x)
def xmap(func, iterable, processes=None):
  with Pool(processes, initializer=worker_init, initargs=(func,)) as p:
    return p.map(worker, iterable)

import numpy as np
from scipy.stats import spearmanr
from scipy import sparse
import dill
import json

def spearman(x,y):
    spear = lambda ft: np.abs(spearmanr(ft.T,y)[0])
    x = zehidense(x)
    re = list(map(spear,x.T))
    return np.array(re)

def zehidense(X):
    if sparse.issparse(X):
        return X.todense()
    if type(X) == np.matrix:
        X = X.A
    return X

def cartesian_product(arrays):
    la = len(arrays)
    dtype = np.find_common_type([a.dtype for a in arrays], [])
    arr = np.empty([len(a) for a in arrays] + [la], dtype=dtype)
    for i, a in enumerate(np.ix_(*arrays)):
        arr[..., i] = a
    return arr.reshape(-1, la)



def np_bool_select(numpi, bools):
    return np.array([x for x,y in zip(zehidense(numpi),bools) if y  ])



def maxnum(X):
    return np.nanmax( np.where(np.isinf(X),-np.Inf,X) )

def minnum(X):
    return np.nanmin( np.where(np.isinf(X),np.Inf,X) )


dumpfile = lambda thing, filename: dill.dump(thing, open(filename, "wb"))
loadfile = lambda filename: dill.load(open(filename, "rb"))

jdumpfile = lambda thing, filename:  open(filename,'w').write(json.dumps(thing))
jloadfile = lambda filename:  json.loads(open(filename,'r').read())


def ndumpfile(thing,filename):
    if type(thing) == list:
        d= { chr(i+98):e  for i,e in enumerate(thing)}
        d['a'] = len(thing)
        np.savez_compressed(filename, **d)
    else:
        np.savez_compressed(filename,a=0,b=thing)

def nloadfile(filename):
    if filename[-4]!= '.':
        print('adding .npz to filename')
        filename += '.npz'

    if filename.endswith('.npy'):
        return np.load(filename)

    # load .npz
    z = np.load(filename, allow_pickle=True)
    num = int(z['a'])
    if num == 0:
        return z['b']
    else:
        return [z[chr(i+98)] for i in range(num) ]


sdumpfile = lambda thing, filename:  sparse.save_npz(filename, thing)
sloadfile = lambda filename:  sparse.load_npz(filename+'.npz')



class spacemap():
    # mapping items to integers...
    def __init__(self, uniqueitems):
        self.itemlist = uniqueitems
        self.integerlist = list(range(len(uniqueitems)))
        self.len = len(uniqueitems)
        self.getitem = {i:k for i,k in enumerate(uniqueitems)}
        self.getint = {k:i for i,k in enumerate(uniqueitems)}

def labelsToIntList(items):
    sm = spacemap(np.unique(items))
    return [sm.getint[i] for i in items], sm

def binarize(X,posratio):
    '''
    lowest posratio -> 1 ;; rest 0
    '''
    argsrt = np.argsort(X)
    if 0< posratio < 1:
        cut = max(int(len(X)*(1-posratio)),1)
    elif len(X) > posratio:
        cut = len(X) - posratio
    else:
        assert False, f'{0<posratio<1= }  {len(X)>posratio= }'

    values = np.ones(len(X), dtype = np.int32)
    values[argsrt[:cut]] = 0
    return values



if __name__ == "__main__":
    a = np.array([0,1,2])
    ndumpfile(a,'adump')
    ndumpfile([a,a],'aadump')
    print(nloadfile('adump'))
    print(nloadfile('aadump'))
    sdumpfile(sparse.csr_matrix(a),'sdump')
    print(sloadfile('sdump'))

    a = np.random.rand(6)
    print(a)
    print(binarize(a,2))




import numpy as np
from sklearn.neighbors import KernelDensity
import multiprocessing as mp
from matplotlib.patches import Ellipse
from lmz import *


def mpmap(func, iterable, chunksize=1, poolsize=5):
    """pmap."""
    pool = mp.Pool(poolsize)
    result = pool.map(func, iterable, chunksize=chunksize)
    pool.close()
    pool.join()
    return list(result)


class TPE:
    '''
    we should discuss how the space is defined:
    space = [(mi,ma),...],{a:(mi,ma)}
    '''
    def __init__(self,f, space, n_init =10, kde_top = 5):

        self.f = f
        self.space = space
        self.n_init = n_init
        self.params = []
        self.values = []
        self.kde_top = kde_top

    def register(self,params:list, values: list):
        self.params+=params
        self.values+= values

    def suggest(self, n= 5):
        if self.n_init > len(self.params):
            return [self.randomsample() for i in range(n)]
        else:
            self.TPEfit()
            return [self.TPEsample() for i in range(n)]

    def randomsample(self):
        # returns 1 random parameter set
        return [np.random.uniform(a,b) for a,b in self.space[0]],\
                   {k:np.random.uniform(a,b) for k,(a,b) in self.space[1].items()}


    def minimize(self,n_iter=20, n_jobs = 5):
        while n_iter > 0:
            args = self.suggest(n_jobs)
            values = mpmap(self.f,args)
            self.register(args, values)
            n_iter -= n_jobs

    ################
    # TPE model
    ##################
    def TPEfit(self):
        assert len(self.params) > 1, 'do some random guessing first'
        #num_v = int(len(self.values)/2)# or self.kde_top
        num_v = self.kde_top
        goodparams = [self.params[i] for i in np.argsort(self.values)[:num_v] ]
        self.tpemodels = [ self._fitkd([l[i] for l,_ in goodparams]) for i in range(len(self.space[0]))]
        #exp = [kd.bandwidth for kd in self.tpemodels]
        #for l,_ in goodparams: plt.Circle(l,exp[0])

    def _fitkd(self, values):
        # n**(-1./(d+4))   heuristic to determine bandwith; n= num_sampled, d = dimensions
        bandwidth = np.std(values)/5 #len(values)**(-1/5)
        print(f"{ bandwidth = }")
        kd = KernelDensity(kernel='gaussian', bandwidth = bandwidth)
        kd.fit(np.array(values).reshape(-1,1))
        return kd

    def TPEsample(self):
        return [kd.sample()[0,0] for kd in self.tpemodels],{}

    ################
    # BAYES model ..
    ##################
    def Bayesfit(self):
        assert len(self.params) > 1, 'do some random guessing first'
        self.baymodels = [ self._fitbayes(s,[(l[i],v) for s,v,(l,_) in zip(self.values,self.params)]) for i,s in enumerate(self.space[0])]

    def _fitbayes(self, space, xyL):
        X,y = Transpose(xyL)
        bayes().fit(space, X,y)

    def Bayessample(self):
        return [bay.sample() for kd in self.baymodels],{}




def myf(args):
    x,y = args[0]
    return abs(x)+abs(y)

if __name__ == "__main__":
    opti = TPE(myf,([(-100,100),(-100,100)],{}),n_init = 10)
    n_iter = 30
    n_jobs = 5
    import matplotlib
    matplotlib.use("module://matplotlib-sixel")
    import matplotlib.pyplot as plt
    while n_iter > 0:
        args = opti.suggest(n_jobs)
        values = mpmap(opti.f,args)
        opti.register(args, values)
        tmp = np.array([a for a,b in args])
        n_iter -= n_jobs
        plt.scatter(tmp[:,0], tmp[:,1])
        plt.show()
        plt.close()
        print(np.mean(values))
    #opti.minimize()
    print ( [p for p,d in opti.params] )
    print (opti.values)



from lmz import *
import numpy as np
import matplotlib.pyplot as plt


'''
1d bayes, interpolating to best expected values between points
'''

def logistic(x):
    x/=2
    return np.exp(x)/(np.exp(x)+1)

def getavgslope(myx, ydata):
    slopes=[]
    for i in Range(1,myx):
        x1 = myx[i-1]
        x2 = myx[i]
        y1 = ydata[i-1]
        y2 = ydata[i]
        slope= abs(y1-y2)/abs(x1-x2)
        slopes.append(slope)
    meanslope = np.mean(slopes)
    return meanslope

def suggest(aruX, aruY,bravery =1 , plot = False):

    aruX, aruY = Transpose(sorted(zip(aruX,aruY)))

    meanslope = getavgslope(aruX, aruY)

    xspan = max(aruX) - min(aruX)
    yspan = max(aruY) - min(aruY)

    newyx = []
    for i in Range(1,myx):
        x1 = aruX[i-1]
        x2 = aruX[i]
        y1 = aruY[i-1]
        y2 = aruY[i]

        slope= (y2-y1)/abs(x1-x2)
        nuxx = x2-x1
        nuxx = logistic(slope/meanslope)*nuxx+x1

        nuyy = max(y1,y2) + bravery*logistic(-abs(slope)/meanslope)*((x2-x1)/(xspan))*yspan

        newyx.append((nuyy,nuxx))

    newyx.sort(reverse = True)
    y,x = Transpose(newyx)
    if plot:
        plt.scatter(aruX, aruY, label = 'given')
        plt.scatter(x,y, label='suggested')
        plt.legend()
        plt.show()
    return x,y



import numpy as np
import structout as so
import matplotlib.pyplot as plt
from lmz import *
from ubergauss.tools import cartesian_product, xmap, maxnum, binarize
from sklearn import ensemble
import umap

'''
We use BORE but make it nicer...
'''





class Minimizer:
    '''
    we should discuss how the space is defined:
    space = [(min,max),...]


    -infer space
    -
    '''


    def __init__(self,f, space = None):
        self.f = f
        self.params = []
        self.values = []
        self.space = space
        self.model= None

    def infer_space(self, data):
        return  [(min(data[:, i]), max(data[:, i])) for i in range(data.shape[1])]

    def init_random(self,n_init):
            pt  = [self.randomsample() for i in range(n_init)]
            scores = xmap(self.f,pt)
            self.register(pt,scores)

    def register(self,params:list, values: list):
        self.params+=params
        self.values+= values

    def randomsample(self):
        return [np.random.uniform(a,b) for a,b in self.space]






    ################
    # Fitting..
    ##################
    def fit(self):
        assert len(self.params) > 1, 'use .register() or init_random() so we have some points to learn from'

        fpos = ensemble.RandomForestClassifier(n_estimators=50)
        X = np.array(self.params)
        Y = binarize(self.values, .25)
        fpos.fit(X, Y)
        self.model = fpos

    def get_grid(self, space, points=100):
        X = [np.linspace(a, b, points) for a, b in space]
        X = cartesian_product(X)
        return X

    def make_sample_terrain(self, points_per_dim= 100):
        '''
        get prediction for the whole space
        '''
        X = self.get_grid(self.space, points_per_dim)
        pos = get1index(self.model)
        YALL = self.model.predict_proba(X)
        Y1, Y2 = YALL[:,pos], YALL[:, 0 if pos == 1 else 0]
        # TODO  Y2 being zero is a problem..
        #  with obvious solutions y2==0 -> picj definitely from there
        ie = Y1.flatten()/Y2.flatten()
        ie = np.where(ie != np.inf, ie, Y1f * maxnum(ie)*2)
        ie*=ie.T # squaring to shift the probabilities around
        cut = np.median(ie)
        X = X[ie > cut]
        ie = ie[ie > cut]
        self.sample_terain = X,ie

    def sample(self, num, draw= False):
        X, w = self.sample_terain
        sample = np.random.choice(Range(X), num, replace=False, p=w / sum(w))
        samples  =  [X[r] for r in sample]

        if draw:
            self.draw_terain_and_samples(self, samples)
        return samples

    def draw_terain_and_samples(self, samples):
            '''
            -> 1d
            -> 2d
            -> do the reduction to 2d

            draw 1.probes pos 2.p/n 3.samples
            '''

            if len(self.space) == 1:
                # known points
                sa, sb = self.space[0]
                so.iprint({p[0]:v for p,v in zip(self.params, self.values)},spacemin = sa, spacemax=sb)
                # terrain
                so.iprint({p[0]:v for p,v in zip(*self.sample_terain)},spacemin = sa, spacemax=sb)
                # sampleeee
                so.iprint({p[0]:1 for p in samples},spacemin = sa, spacemax=sb)
                return

            X = self.params
            BG = self.sample_terain[0]
            if len(self.space) > 2:
                p = umap.UMAP()
                X = p.fit_transform(X, self.values)
                BG = p.transorm(BG)
                samples = p.transform(samples)

            # plot known points
            xylim = self.infer_space(BG)
            plt.figure(figsize=(9,3))
            plt.subplot(131)
            d2plot(X,self.values, title = 'sampled data', xylim = xylim)
            # plot terain
            plt.subplot(132)
            d2plot(BG,self.sample_terain[1], title = 'terrain', xylim = xylim)
            plt.subplot(133)
            d2plot(X,y='r', title = 'new samples', xylim = xylim)
            plt.show()
            plt.close()


    def minimize(self,n_iter=20, n_jobs = 5):
        while n_iter > 0:
            args = self.suggest(n_jobs)
            values = xmap(self.f,args)
            self.register(args, values)
            n_iter -= n_jobs



    def bestsuggestion(self):
        # so we sample,,, ie are the weights...
        #return random.choices(self.model[0], self.model[1])[0]
        X,w = self.model
        sample = np.argmax(w)
        return X[sample]



def get1index(clf):
    if clf.classes_[0] == 1:
        return 0
    else:
        return 1

def d2plot(X,y, done = False, title = None,xylim=None):
    pl=plt.scatter(X[:,0],X[:,1], c= y,s=9)
    plt.xlim(*xylim[0])
    plt.ylim(*xylim[1])
    plt.colorbar(pl)
    if title:
        plt.title(title)
    if done:
        plt.show()
        plt.close()




def myf(args):
    #return (args[0]**2)+(args[1]**2)
    return min(((args[0]+30)**2)+((args[1]+90)**2), ((args[0]-30)**2)+((args[1]-60)**2))




if __name__ == "__main__":

    import matplotlib

    matplotlib.use("module://matplotlib-sixel")

    opti = Minimizer(myf,[(-100,100),(-100,100)])

    X = opti.get_grid(opti.space)
    plt.scatter(X[:,0],X[:,1], c=Map(myf, X), s=9)
    plt.show();plt.close()

    if True:
        for i in range(6):
            pt = opti.fit(draw=True, sample = 5)
            score = [myf(x) for x in pt]
            opti.register(pt,score)

    if False:
        for i in range(10):
            pt  = opti.bestsuggestion()
            score = [myf(pt)]
            opti.register([pt],score)
            opti.fit(draw=True)



import random
from scipy.special import softmax
import numpy as np
import multiprocessing as mp
from matplotlib.patches import Ellipse
from lmz import *
import forest

'''
ok TPE is terrible so lets  go to bayesian...

- we have a function and a space to optimize...
- fit the trees with variance estimation
- predict...
- draw
- sample
- repeat
'''

def mpmap(func, iterable, chunksize=1, poolsize=5):
    """pmap."""
    pool = mp.Pool(poolsize)
    result = pool.map(func, iterable, chunksize=chunksize)
    pool.close()
    pool.join()
    return list(result)


class BAY:
    '''
    we should discuss how the space is defined:
    space = [(mi,ma),...],{a:(mi,ma)}
    '''
    def __init__(self,f, space, n_init =10):

        self.f = f
        self.space = space
        self.params = []
        self.values = []

        if n_init:
            pt  = [self.randomsample() for i in range(n_init)]
            scores = mpmap(f,pt)
            self.register (pt,scores)



    def register(self,params:list, values: list):
        self.params+=params
        self.values+= values

    def suggest(self, n= 5):
        pass

    def randomsample(self):
        # returns 1 random parameter set
        return [np.random.uniform(a,b) for a,b in self.space]


    def minimize(self,n_iter=20, n_jobs = 5):
        while n_iter > 0:
            args = self.suggest(n_jobs)
            values = mpmap(self.f,args)
            self.register(args, values)
            n_iter -= n_jobs

    ################
    # TPE model
    ##################
    def fit(self, sample=0, draw=False):
        assert len(self.params) > 1, 'do some random guessing first'

        if 'model'  in self.__dict__:
            #self.space= [(x.min(),x.max()) for x in self.model[0].T]
            pass

        f = forest.RandomForestRegressor(n_estimators = 50)
        f.fit(np.array(self.params), self.values)

        '''
        get prediction for the whole space
        '''
        X = [np.linspace(a,b,100) for a,b in self.space]
        X = cartesian_product(X)
        Y,S = f.predict(X, return_std= True)

        ie = Y-1*S # expected valies

        cut = np.median(ie)
        XNU = X[ie < cut]
        ieNU = ie[ie < cut]-cut
        ieNU = -ieNU
        ieNU/=max(ieNU)
        ieNU = ieNU*ieNU.T
        self.model = XNU, ieNU

        probe_here = 0
        if sample:
            probe_here = self.sample(sample)

        if draw:
            plt.figure(figsize=(9,6))

            plt.subplot(231)
            d2plot(np.array(self.params),self.values, title = 'sampled data')
            plt.subplot(232)
            d2plot(X,np.log2(Y), title = 'log predict')
            plt.subplot(233)
            d2plot(X,S, title = 'std')
            plt.subplot(234)
            d2plot(X,np.log(ie), title = 'log prediction - std')

            plt.subplot(235)
            d2plot(XNU,ieNU, title = 'sampleweight')

            if sample:
                plt.subplot(236)
                d2plot(np.array(probe_here),y='r', title = 'new samples')
            plt.show()
            plt.close()
        return probe_here


    def sample(self, num):
        # so we sample,,, ie are the weights...
        #return random.choices(self.model[0], self.model[1])[0]
        X,w = self.model
        sample = np.random.choice(Range(X),num,replace=False, p=w/sum(w))
        return [X[r] for r in sample]


    def bestsuggestion(self):
        # so we sample,,, ie are the weights...
        #return random.choices(self.model[0], self.model[1])[0]
        X,w = self.model
        sample = np.argmax(w)
        return X[sample]


def cartesian_product(arrays):
    la = len(arrays)
    dtype = np.find_common_type([a.dtype for a in arrays], [])
    arr = np.empty([len(a) for a in arrays] + [la], dtype=dtype)
    for i, a in enumerate(np.ix_(*arrays)):
        arr[..., i] = a
    return arr.reshape(-1, la)


import matplotlib
matplotlib.use("module://matplotlib-sixel")
import matplotlib.pyplot as plt

def d2plot(X,y, done = False, title = None):
    pl=plt.scatter(X[:,1],X[:,0], c= y,s=9)
    plt.xlim(-100,100)
    plt.ylim(-100,100)
    plt.colorbar(pl)
    if title:
        plt.title(title)
    if done:
        plt.show()
        plt.close()

def myf(args):
    return abs(args[0]**2)+abs(args[1]**2)




if __name__ == "__main__":
    opti = BAY(myf,[(-100,100),(-100,100)],n_init = 5)


    if True:
        for i in range(5):
            pt = opti.fit(draw=True, sample = 3)
            score = [myf(x) for x in pt]
            opti.register(pt,score)

    if False:
        for i in range(10):
            pt  = opti.bestsuggestion()
            score = [myf(pt)]
            opti.register([pt],score)
            opti.fit(draw=True)
